function [data, status] = get_ncfile ( ncfile )
% GET_NCFILE:  read the contents of a netcdf file into a nested structure
%
% USAGE:  struct = get_ncfile ( ncfile );
%
% PARAMETERS:
% ncfile:  
%    Input netcdf file name.
% struct:
%    Output.  Structure with all netcdf data present.  Each variable will
%    be a substructure.  Within each variable lies the data and each 
%    attribute.  The global attributes are stored in a sub-structure
%    called 'global_atts'
%







%
% assume failure until success is known
status = -1;
data = [];


% Show usage if too few arguments.
%
if nargin~=1 
   help get_ncfile;
   return;
end


%
% Open netCDF file
%
[cdfid,rcode ]=ncmex('open',ncfile,'NOWRITE');
if cdfid < 0
	fprintf ( 2, '%s:  ncmex:open failed on %s.\n', mfilename, ncfile );
	return
end

% Suppress warning messages from netCDF
[rcode]=ncmex('setopts',0);


[ndims, nvars, ngatts, recdim, status] = ncmex('INQUIRE', cdfid);
if status < 0
	fprintf ( 2, '%s:  ncmex:inquire failed on %s.\n', mfilename, ncfile );
	ncmex('close',cdfid);
    return
end

for varid=0:nvars-1

	varstruct = [];

	[varname, datatype, ndims, dims, natts, status] = ncmex('VARINQ', cdfid, varid);
	if status < 0 
		fprintf ( 2, '%s:  ncmex:varinq failed on varid %d, file %s.\n', mfilename, varid, ncfile );
		ncmex('close',cdfid);
	end

	[values, status] = ncvarget(ncfile, varname);
	if status < 0 
		fprintf ( 2, '%s:  ncvarget failed on %s, file %s.\n', mfilename, varname, ncfile );
		ncmex('close',cdfid);
	end

	varstruct = setfield ( varstruct, 'data', values );
%	command = sprintf ( 'data.%s.data = values;', varname );
%	eval (command );



	%
	% get all the attributes
	for attnum = 0:natts-1

		[attname, status] = ncmex('ATTNAME', cdfid, varid, attnum);
		if status < 0 
			fprintf ( 2, '%s:  ncmex:attname failed on %s, file %s.\n', mfilename, varname, ncfile );
			ncmex('close',cdfid);
		end

		[attval, status] = ncattget(ncfile, varname, attname);
		if status < 0 
			fprintf ( 2, '%s:  ncattget failed on %s, %s, file %s.\n', mfilename, attname, varname, ncfile );
			ncmex('close',cdfid);
		end
		
		%
		% Matlab structures don't like the leading '_'
		if strcmp(attname,'_FillValue' )
			attname = 'FillValue';
		end

		%
		% this puts the attribute into the variable structure
		varstruct = setfield ( varstruct, attname, attval );
%		command = sprintf ( 'data.%s.%s = attval;', varname, attname );
%		eval (command );


	end


	%
	% Add this variable to the entire file structure
	data = setfield ( data, varname, varstruct );

end


%
% Do the same for the global attributes
%
% get all the attributes
varname = 'global';
global_atts = [];
for attnum = 0:ngatts-1

	[attname, status] = ncmex('ATTNAME', cdfid, varname, attnum);
	if status < 0 
		fprintf ( 2, '%s:  ncmex:attname failed on %s, file %s.\n', mfilename, varname, ncfile );
		ncmex('close',cdfid);
	end

	[attval, status] = ncattget(ncfile, varname, attname);
	if status < 0 
		fprintf ( 2, '%s:  ncattget failed on %s, %s, file %s.\n', mfilename, attname, varname, ncfile );
		ncmex('close',cdfid);
	end
	
	global_atts = setfield ( global_atts, attname, attval );
%	command = sprintf ( 'data.global_atts.%s = attval;', attname );
%	eval (command );


end

data = setfield ( data, 'global_atts', global_atts );

ncmex('close',cdfid);

status = 1;

return
