function status = nc_add_recs ( ncfile, new_data )
% NC_ADD_RECS:  add records onto the end of a netcdf file
%
% USAGE:  status = nc_add_recs ( ncfile, new_data );
% 
% INPUT:
%   ncfile:  netcdf file
%   new_data:  Matlab structure.  Each field is a data array
%      to be written to the netcdf file.  Each array had
%      better be the same length.  All arrays are written
%      in the same fashion.
%     
% OUTPUT:
%   status:  -1 if the routine fails
%
% AUTHOR: 
%   johnevans@acm.org
%





%
% Open netCDF file
%
[cdfid,rcode ]=ncmex('open',ncfile,'NOWRITE');
if cdfid < 0
   disp(['Error in ncmex: Can''t open ' ncfile]);
   return
end

% Suppress warning messages from netCDF
[rcode]=ncmex('setopts',0);

%
% Get the dimension info for time
[dim_name,dim_size,rcode]=ncmex('diminq',cdfid,'time');
time_corner(1) = dim_size;
time_count(1) = size(new_data.time,1);

%
% For each field of "new_data" buffer, inquire as to the dimensions in the
% NetCDF file.  We need this data to properly tell nc_varput how to write
% the data
input_variable = fieldnames ( new_data );
num_vars = length(input_variable);
varsize = [];
for j = 1:num_vars
	dimsize = [];
	[varid, rcode] = ncmex('VARID', cdfid, input_variable{j} );
	if ( rcode == -1 )
		fprintf ( 2, '%s:  ncmex::varid failed on %s, %s.\n', mfilename, input_variable{j}, ncfile );
		status = -1;
		ncmex('close',cdfid);
		return
	end

	[varname, datatype, ndims, dimids, natts, status] = ncmex('VARINQ', cdfid, varid);
	if ( rcode == -1 )
		fprintf ( 2, '%s:  ncmex::varinq failed on %s, %s.\n', mfilename, input_variable{j}, ncfile );
		status = -1;
		ncmex('close',cdfid);
		return
	end

	for k = 1:ndims
		[dimname, dim_length, status] = ncmex('DIMINQ', cdfid, dimids(k) );
		if ( rcode == -1 )
			fprintf ( 2, '%s:  ncmex::diminq failed on dimid %d, variable %s, %s.\n', mfilename, dimid, input_variable{j}, ncfile );
			status = -1;
			ncmex('close',cdfid);
			return
		end
		dimsize(k) = dim_length;
	end

	varsize = setfield ( varsize, input_variable{j}, dimsize);

end

status = ncmex('close',cdfid);
if status == -1
	fprintf ( 2, '%s:  ncmex::close failed on %s.\n', mfilename, ncfile );
	return
end


%
% Assume failure until we know otherwise.
status = -1;



%
% write out each data field, as well as the minimum and maximum
for i = 1:num_vars

    current_var = input_variable{i};
	fprintf ( 1, '%s:  processing %s...\n', mfilename, current_var );

    current_var_data = getfield ( new_data, current_var );
	var_buffer_size = size(current_var_data);

	netcdf_var_size = getfield ( varsize, current_var );

	corner = zeros( 1, length(netcdf_var_size) );
	count = ones( 1, length(netcdf_var_size) );

	corner(1) = time_corner(1);
	count(1) = time_count(1);


	
	for j = 2:length(var_buffer_size(:))
		if ( var_buffer_size(j) > 1 )
			count(j) = var_buffer_size(j);
		end
	end



    status = ncvarput ( ncfile, current_var, current_var_data, corner, count );
    if ( status == -1 )
        fprintf ( 2, '%s:  ncvarput failed on %s:%s\n', mfilename, ncfile, current_var );
        return
    end
   

end


%
% If here, we know we did ok
status = 1;

return





