function [depth, my_status] = nc_getdepth ( ncfile, varname )
% NC_GETDEPTH:  retrieve the depth variariable defined for "varname"
%
% USAGE:  [depth, status] = nc_getdepth ( ncfile, varname );
%
% PARAMETERS:
% ncfile:  
%    Input netcdf file name.
% varname:  
%    variable for which we desire depth
%
% This can be kind of tricky.  We handle it by looking at the dimensions.
% If a dimension is just "depth", then that is good enough.  If it has
% the word "depth" in it, that is also good enough.  If it has an epic
% code that is 3 (the epic code for depth) that is also good enough.
% The dimension must have an associated coordinate variable.  Otherwise,
% tough cookies.




% assume false until we know otherwise
depth = [];
my_status = -1;


% Show usage if too few arguments.
%
if nargin~=2 
   help nc_getdepth;
   return;
end


%
% Open netCDF file
%
[cdfid,rcode ]=ncmex('open',ncfile,'NOWRITE');
if cdfid < 0
	fprintf ( 2, '%s:  ncmex:open failed on %s.\n', mfilename, ncfile );
	return
end

% Suppress warning messages from netCDF
[rcode]=ncmex('setopts',0);

[varid, rcode] = ncmex('VARID', cdfid, varname);
if rcode < 0
	fprintf ( 2, '%s:  ncmex:varid failed on %s, %s.\n', mfilename, varname, ncfile );
	ncmex('close',cdfid);
    return
end


[varname, datatype, ndims, dims, natts, status] = ncmex('VARINQ', cdfid, varname);
if status < 0 
	fprintf ( 2, '%s:  ncmex:varinq failed on varid %d, file %s.\n', mfilename, varid, ncfile );
	ncmex('close',cdfid);
	return
end




%
% Look at each dimension
for j = 1:length(dims)

	theDimid = dims(j);

	[dim_name, dim_length, status] = ncmex('diminq', cdfid, theDimid);
	if status < 0 
		fprintf ( 2, '%s:  ncmex:diminq failed on varid %d, file %s.\n', mfilename, varid, ncfile );
		ncmex('close',cdfid);
		return
	end


	%
	% what is the coordinate variable corresponding to this dimension?
	itis = nc_iscoordvar ( ncfile, dim_name );
	itisnot = 1 - itis;
	if itisnot
		continue;
	end

	
	%
	% Does the dimension have an attribute called "standard_name"?  If so, is it "depth"?
	% If so, that works.
	% This is a CF convention.
	[datatype, len, status] = ncmex('ATTINQ', cdfid, dim_name, 'standard_name');
	if ( status ~= -1 )
		[standard_name, status, msg] = nc_attget ( ncfile, dim_name, 'standard_name' );
		if ( status ~= -1 )
			if ( strcmp(standard_name,'depth') )
				[depth, my_status] = nc_varget ( ncfile, dim_name );
				ncmex('close',cdfid);
				return
			end
		end

	end

	%
	% Is the dimension name "depth"?  If so, that's good enough for me.
	if ( strcmp(dim_name,'depth') )
		[depth, my_status] = nc_varget ( ncfile, dim_name );
		ncmex('close',cdfid);
		return
	end



	%
	% Does it have an epic code?  If so, is it three?  That's the epic code for depth.
	[datatype, len, status] = ncmex('ATTINQ', cdfid, dim_name, 'epic_code');
	if ( status ~= -1 )
		[epic_code, status, msg] = nc_attget ( ncfile, dim_name, 'epic_code' );
		if ( status ~= -1 )
			if ( epic_code == 3 )
				[depth, my_status] = nc_varget ( ncfile, dim_name );
				ncmex('close',cdfid);
				return
			end
		end

	end

	%
	% Is the dimension name even have "depth" in it?  If so, that's good enough for me.
	if ( ~isempty(findstr(dim_name,'depth')) )
		[depth, my_status] = nc_varget ( ncfile, dim_name );
		ncmex('close',cdfid);
		return
	end



end

ncmex('close',cdfid);


%
% Ok so none of the named dimensions appeared to  have depth in them.
% The CF standard calls for a "coordinates" attribute.  This should be
% a space delimited attribute with a list of spatio-temporal list of
% dimensions.  Check them.
[coordinates, status, msg] = nc_attget ( ncfile, varname, 'coordinates' );
if ( status ~= -1 )
	r = coordinates;
	while 1
		[t,r] = strtok ( coordinates );
		if isempty(t)
			break;
		end
		[cdepth,cstatus] = nc_getdepth ( ncfile, t );
		if ( cstatus == 1 )
			depth = cdepth;
			my_status = cstatus;
			return;
		end
	end
end



%
% Last chance.  There may be a "sensor_depth" attribute.
[sensor_depth, status, msg] = nc_attget ( ncfile, varname, 'sensor_depth' );
if ( status == -1 )
	my_status = -1;
	return;
end
depth = sensor_depth;

my_status = 1;

return;


