function [dstruct, status] = nc_getinfo ( ncfile )
% NC_GETINFO:  read the metadata of a netcdf file
%
% USAGE:  [struct, info] = nc_getinfo ( ncfile );
%
% PARAMETERS:
% Input:
%    ncfile:  
%       Input netcdf file name.
% Output:
%    struct:
%       Output.  Structure with all netcdf data present.  
%       The structure has a field for each netcdf variable, plus
%       one for the global attributes.  Within each variable field
%       is a field for the size of the data (derived from each
%       dimension), the name of the dimensions upon which it is
%       defined, and a structure of attribute values.
%    status:
%       -1 for failure
%
% AUTHOR:
%    johnevans@acm.org
%







%
% assume failure until success is known
status = -1;
dstruct = [];


% Show usage if too few arguments.
%
if nargin~=1 
   help nc_getinfo;
   return;
end


%
% Open netCDF file
%
[cdfid,rcode ]=ncmex('open',ncfile,'NOWRITE');
if cdfid < 0
	fprintf ( 2, '%s:  ncmex:open failed on %s.\n', mfilename, ncfile );
	return
end

% Suppress warning messages from netCDF
[rcode]=ncmex('setopts',0);


[ndims, nvars, ngatts, recdim, status] = ncmex('INQUIRE', cdfid);
if status < 0
	fprintf ( 2, '%s:  ncmex:inquire failed on %s.\n', mfilename, ncfile );
	ncmex('close',cdfid);
    return
end


%
% retrieve the sizes of each dimension
for dimid = 0:ndims-1
	[dimname{dimid+1}, dimlength(dimid+1), status] = ncmex('DIMINQ', cdfid, dimid);
	if status < 0
		fprintf ( 2, '%s:  ncmex:inquire failed on %s.\n', mfilename, ncfile );
		ncmex('close',cdfid);
		return
	end
end



for varid=0:nvars-1

	varstruct = [];

	[varname, datatype, ndims, dims, natts, status] = ncmex('VARINQ', cdfid, varid);
	if status < 0 
		fprintf ( 2, '%s:  ncmex:varinq failed on varid %d, file %s.\n', mfilename, varid, ncfile );
		ncmex('close',cdfid);
	end

	%
	% set the dimension names for this particular variable
	varstruct = setfield ( varstruct, 'dimensions', dimname(dims+1) );

	%
	% Set the size of the data
	varstruct = setfield ( varstruct, 'data_size', dimlength(dims+1) );


	%
	% get all the attributes
	attributes = [];
	for attnum = 0:natts-1

		[attname, status] = ncmex('ATTNAME', cdfid, varid, attnum);
		if status < 0 
			fprintf ( 2, '%s:  ncmex:attname failed on %s, file %s.\n', mfilename, varname, ncfile );
			ncmex('close',cdfid);
		end

		[attval, status] = ncattget(ncfile, varname, attname);
		if status < 0 
			fprintf ( 2, '%s:  ncattget failed on %s, %s, file %s.\n', mfilename, attname, varname, ncfile );
			ncmex('close',cdfid);
		end
		
		%
		% Matlab structures don't like the leading '_'
		if strcmp(attname,'_FillValue' )
			attname = 'FillValue';
		end

		%
		% could the attribute name  be interpreted as a number?
		% If so, must fix this.
		% An attribute name of, say, '0' is not permissible in matlab
		if ~isnan(str2double(attname))
			stored_attname = ['x_' attname];
		else
			stored_attname = attname;
		end

		%
		% this puts the attribute into the variable structure
		attributes = setfield ( attributes, stored_attname, attval );


	end


	%
	% add the attributes to the variable structure
	varstruct = setfield ( varstruct, 'attributes', attributes );


	%
	% Add this variable to the entire file structure
	dstruct = setfield ( dstruct, varname, varstruct );

end


%
% Do the same for the global attributes
%
% get all the attributes
varname = 'global';
global_atts = [];
for attnum = 0:ngatts-1

	[attname, status] = ncmex('ATTNAME', cdfid, varname, attnum);
	if status < 0 
		fprintf ( 2, '%s:  ncmex:attname failed on %s, file %s.\n', mfilename, varname, ncfile );
		ncmex('close',cdfid);
	end

	[attval, status] = ncattget(ncfile, varname, attname);
	if status < 0 
		fprintf ( 2, '%s:  ncattget failed on %s, %s, file %s.\n', mfilename, attname, varname, ncfile );
		ncmex('close',cdfid);
	end
	
	global_atts = setfield ( global_atts, attname, attval );
%	command = sprintf ( 'data.global_atts.%s = attval;', attname );
%	eval (command );


end

dstruct = setfield ( dstruct, 'global_atts', global_atts );

ncmex('close',cdfid);

status = 1;

return
