function [varstruct, status] = nc_getvarinfo ( ncfile, varname )
% NC_GETINFO:  read the metadata of a netcdf file
%
% USAGE:  [struct, status] = nc_getvarinfo ( ncfile, varname );
%
% PARAMETERS:
% Input:
%    ncfile:  
%       Input netcdf file name.
%    varname:
%       Name of the variable of whose metadata we wish to retrieve.
% varstruct:
%    Output.  
%       varstruct:  Structure with all netcdf variable data present.  
%       status:  -1 if failure.  Usually if a variable does not exist
%
% AUTHOR:
%    johnevans@acm.org
%







%
% assume failure until success is known
status = -1;
varstruct = [];


% Show usage if too few arguments.
%
if nargin~=2 
   help nc_getvarinfo;
   return;
end


%
% Open netCDF file
%
[cdfid,rcode ]=ncmex('open',ncfile,'NOWRITE');
if cdfid < 0
	fprintf ( 2, '%s:  ncmex:open failed on %s.\n', mfilename, ncfile );
	return
end

% Suppress warning messages from netCDF
[rcode]=ncmex('setopts',0);


[ndims, nvars, ngatts, recdim, status] = ncmex('INQUIRE', cdfid);
if status < 0
	ncmex('close',cdfid);
    return
end


%
% retrieve the sizes of each dimension
for dimid = 0:ndims-1
	[dimname{dimid+1}, dimlength(dimid+1), status] = ncmex('DIMINQ', cdfid, dimid);
	if status < 0
		fprintf ( 2, '%s:  ncmex:inquire failed on %s.\n', mfilename, ncfile );
		ncmex('close',cdfid);
		return
	end
end


[varid, rcode] = ncmex('VARID', cdfid, varname);
if ( rcode < 0 )
	fprintf ( 2, '%s:  ncmex:inquire failed on variable %s, file %s.\n', mfilename, varname, ncfile );
	ncmex('close',cdfid);
	return
end

	varstruct = [];

	[varname, datatype, ndims, dims, natts, status] = ncmex('VARINQ', cdfid, varid);
	if status < 0 
		fprintf ( 2, '%s:  ncmex:varinq failed on varid %d, file %s.\n', mfilename, varid, ncfile );
		ncmex('close',cdfid);
	end

	%
	% set the dimension names for this particular variable
	varstruct = setfield ( varstruct, 'dimensions', dimname(dims+1) );

	%
	% Set the size of the data
	varstruct = setfield ( varstruct, 'data_size', dimlength(dims+1) );


	%
	% get all the attributes
	attributes = [];
	for attnum = 0:natts-1

		[attname, status] = ncmex('ATTNAME', cdfid, varid, attnum);
		if status < 0 
			fprintf ( 2, '%s:  ncmex:attname failed on %s, file %s.\n', mfilename, varname, ncfile );
			ncmex('close',cdfid);
		end

		[attval, status] = ncattget(ncfile, varname, attname);
		if status < 0 
			fprintf ( 2, '%s:  ncattget failed on %s, %s, file %s.\n', mfilename, attname, varname, ncfile );
			ncmex('close',cdfid);
		end
		
		%
		% Matlab structures don't like the leading '_'
		if strcmp(attname,'_FillValue' )
			attname = 'FillValue';
		end

		%
		% this puts the attribute into the variable structure
		attributes = setfield ( attributes, attname, attval );


	end


	%
	% add the attributes to the variable structure
	varstruct = setfield ( varstruct, 'attributes', attributes );


ncmex('close',cdfid);

status = 1;

return
