function status = nc_tscopy ( input_file, output_file, arg3, arg4 )
% NC_TSCOPY:  Copies first few time series variables of a netcdf file
%
% Before using this function, you need to do a little work at the
% command line prompt.  First make a cdl file from the old file.
%
% > ncdump -h input_file > input_file.cdl
%
% Then edit this cdl file to make the number of time elements 0.
% Before doing this, the header should look something like
%   
%   netcdf old {
%   dimensions:
%   	lon = 1 ;
%   	lat = 1 ;
%   	depth = 30 ;
%   	time = UNLIMITED ; // (13180 currently)
%   
%
% Edit it to look like the following
%   netcdf old {
%   dimensions:
%   	lon = 1 ;
%   	lat = 1 ;
%   	depth = 30 ;
%   	time = UNLIMITED ; // (0 currently)
%   
%
% Then recreate the skeleton new netcdf file with the command
% > ncgen -o output_file input.cdl
%
% This results in a brand new netcdf file with no data in it.
% Now use this script in matlab to copy the first num_rec 
% elements over.
% 
% USAGE:  status = nc_tscopy ( input_file, output_file, num_recs );
%
% or:     status = nc_tscopy ( input_file, output_file, jd1, jd2 );
%
% PARAMETERS:
%   Input:
%      input_file:  old netcdf file you wish to copy
%      output_file: new netcdf file
%      num_recs:  number of time records from the old netcdf file
%                 you wish to have placed in the new netcdf file
%      jd1: julian day number of starting time
%      jd2: julian day number of stopping time
%   Output:
%      status:  -1 if failure
%



status = -1;

if nargin < 3
	help nc_tscopy;
	return
end
if nargin == 3
	start = 1;
	stop = arg3;
end

if nargin == 4

	start = 1;
	stop = arg3;

	%
	% get the time
	[time, status] = ncvarget ( input_file, 'time' );
	if status == -1
		fprintf ( 2, '%s:  ncvarget failed on %s\n', mfilename, input_file);
		return
	end

	ind = find ( (time>=arg3) & (time<=arg4) );
	if isempty(ind)
		fprintf ( 2, ...
		          '%s:  No time datums found in range %s thru %s in %s\n', ...
		          mfilename, ...
				  input_file, ...
				  jd2gdtstamp(arg3), ...
				  jd2gdtstamp(arg4), ...
				  input_file );
		return
	end

	start = ind(1);
	stop = ind(end);
	
end





varname_struct = ncgetvarnames ( input_file );
varnames = fieldnames ( varname_struct );
num_vars = length(varnames);

old_ncopts = ncmex('SETOPTS',0);


%
% open the input netcdf file to query the time dimension
cdfid = ncmex('OPEN', input_file, 'read');
if cdfid == -1
	fprintf ( 2, '%s:  ncmex:open failed on %s\n', mfilename, input_file);
	return
end


[time_dimid, status] = ncmex('DIMID', cdfid, 'time');
if status == -1
	fprintf ( 2, '%s:  ncmex:dimid failed on %s\n', mfilename, input_file);
	return
end


%
% loop thru each variable
for j = 1:num_vars
	[data, status] = ncvarget ( input_file, varnames{j} );
	if status == -1
		fprintf ( 2, '%s:  ncvarget failed on %s:%s\n', mfilename, input_file, varnames{j} );
		return
	end

	varnames{j}
	size(data)

	%
	% get the variable's dimension ids
	[var_name_dummy, datatype, ndims, dimids, natts, status] = ncmex('VARINQ', cdfid, varnames{j});
	if status == -1
		fprintf ( 2, '%s:  ncmex:varinq failed on %s:%s\n', mfilename, input_file, varnames{j} );
		return
	end


	%
	% does it depend upon time?
	pos = find(dimids==time_dimid);
	if isempty(pos)
		%
		% no, copy the whole thang
		;
	else
		
		[r,c] = size(data);
		if ( c > 1 )
			data = data(start:stop,:); 
		else 
			data = data(start:stop); 
		end

	end


	size(data)
	status = ncvarput ( output_file, varnames{j}, data );
	if status == -1
		fprintf ( 2, '%s:  ncvarput failed on %s:%s\n', ...
		          mfilename, ...
				  output_file, ...
				  varnames{j} );
		return
	end



end

status = ncmex ( 'close', cdfid );
if status == -1
	fprintf ( 2, '%s:  ncmex:close failed on %s\n', mfilename, input_file);
	return
end

status = 1;

return
