function [values, status] = nc_varget(cdf, var, corner,count)
% NC_VARGET Get a NetCDF hyperslab.
%  NC_VARGET('CDF','VAR',[COORD],[COUNT]) returns the hyperslab 
%   from NetCDF file 'CDF, variable 'VAR', starting at corner 
%   [COORD] and containing [COUNT] elements along the edges.  
%   The starting index of an array is 0 (C-Language convention).
%
% Example:
%    [values, status] = nc_varget('foo.cdf', 'x', [0 0], [2 3])
%
% PARAMETERS:
%    values:  return data
%    status:  -1 if function failed, 0 if ok
%    CDF:  name of netcdf file
%    VAR:  name of variable to retrieve
%    COORD:  starting coordinate of the retrieval
%    COUNT:  length of the retrieval along each dimension





%
% assume failure until success is known
values = [];
status = -1;


% Show usage if too few arguments.
%
if nargin~=2 & nargin~=4,
   help nc_varget;
   return;
end


%
% Open netCDF file
%
[cdfid,rcode ]=ncmex('open',cdf,'NOWRITE');
if cdfid < 0
   disp(['Error in ncmex: Can''t open ' cdf]);
   return
end

% Suppress warning messages from netCDF
[rcode]=ncmex('setopts',0);

%
% Get variable id
%
[varid]=ncmex('varid',cdfid,var);
if varid < 0
   disp(['Error in ncmex: Can''t get variable ' var ' in file ' cdf]);
   return
end
[var_name,var_type,nvdims,var_dim,natts]=ncmex('varinq',cdfid,varid);


%
% Automagically get all the data if slab is not specified
%
%if nargin==2,
  for n=1:nvdims,
    dimid=var_dim(n);
    [dim_name,dim_size,rcode]=ncmex('diminq',cdfid,dimid);
    the_corner(n)=0;
    the_count(n)=dim_size;
  end
the_var_size = the_count;
%end


%
% If restrictions were specified, set them here.
if ( nargin ~= 2 )
	
  for n=1:length(count)
    the_corner(n)=corner(n);
    the_count(n)=count(n);
  end

end



%
% Is the requested size negative?
ind = find(the_count<0);
if ~isempty(ind)
	values = [];
	hyperslab_start = sprintf ( '[%d]', the_corner );
	hyperslab_request = sprintf ( '[%d]', the_count );
	hyperslab_extent = sprintf ( '[%d]', the_var_size );
	fprintf ( 2, '%s:  requested %s data hyperslab extent %s + %s is negative, restate to fit existing data bounds %s.\n', mfilename, var, hyperslab_start, hyperslab_request, hyperslab_extent );
	ncmex('close',cdfid);
	status = -1;
	return
end



%
% Does the hyperslab exceed the dataset size?
ind = find((the_corner+the_count) > the_var_size );
if ~isempty(ind)
	values = [];
	hyperslab_start = sprintf ( '[%d]', the_corner );
	hyperslab_request = sprintf ( '[%d]', the_count );
	hyperslab_extent = sprintf ( '[%d]', the_var_size );
	fprintf ( 2, '%s:  requested %s data hyperslab %s + %s exceeds existing data bounds %s.\n', mfilename, var, hyperslab_start, hyperslab_request, hyperslab_extent );
	ncmex('close',cdfid);
	status = -1;
	return
end


%
% Get slab, unless the count is zero.
%
% There is a bug in Mexcdf.  If the count of the
% first dimension is zero, it core dumps.  Something to figure
% out later.
if prod(the_count) == 0
	values = [];
	ncmex('close',cdfid);
	status = 1;
	return
end



%
% want slowest varying dimension to be # of rows
[values,rcode] = ncmex('varget',cdfid,varid,the_corner,the_count,1);

%
% If it's a 1D vector, make it a column vector.
if length(the_corner) == 1
	values = values(:);
elseif ( length(the_corner) > 1 )
	pv = fliplr ( [1:length(the_corner)] );
	values = permute(values,pv);
end                                                                                   

%
% Get the fill value, if present.
% Because ncmex automatically handles the scale_factor and
% add_offset before returning (bad, bad, bad), we have to 
% retrieve these first and look to see where the scaled returned
% data equals the scaled fill value.
[scale, status] = ncattget ( cdf, var, 'scale_factor' );
if status == -1 
	% For some reason 'attget' does not return -1 in 'status' if
	% it fails.  Check the length(scale).  If larger than 1, then
	% we know there is a problem.
	scale = 1;
end
[offset, status] = ncattget ( cdf, var, 'add_offset' );
if status == -1 
	offset = 0;
end

[fv, status] = ncattget ( cdf, var, '_FillValue' );
if ( status ~= -1  )
	sc_fv = fv*scale+offset;
	ind = find(values==sc_fv);
	values(ind) = NaN;
end

%
% Look for the valid range attribute.  If any data values are
% outside of the valid range, set them to NaN.  Once again, we
% have to worry about the values being scaled too soon.
%[valid_range, status] = ncattget(cdf, var, 'valid_range');  
%if status == -1 
%	valid_range = [-1e308 1e308];
%end
%scaled_valid_range = valid_range * scale + offset;
%
%ind = find(   values < scaled_valid_range(1) ...
%            | values > scaled_valid_range(2) );
%values(ind) = NaN;
%

ncmex('close',cdfid);

status = 1;
