function status = nc_varput(ncfile, variable, data, start,count)
% NCVARPUT: Put a NetCDF hyperslab into a NetCDF file.
%
% USAGE:  status = nc_varput(file, var, data, start, count );
%
% PARAMETERS:
%   status:  -1 for failure, 0 for success
%   
% Example:
%    status = nc_varput('foo.cdf', 'x', data, [0 0], [2 3])
%    file:  name of input netcdf file
%    var:  name of variable to write to
%    data:  to be written to netcdf file
%    start:  coordinates where we start writing to.
%            Optional.  If not supplied, assume [0,..]
%    count:  how many values to write along each dimension
%            Optional.  If not supplied, assume as much
%            as can be written.
%
% Currently, the proper error status flag is NOT set if the
% 'varput' operation fails due to improper "start" or "count"
% arguments.  If no error message is given but the data has
% obviously not be written, check that.






%
% Assume failure until told otherwise.
status = -1;

%
% Suppress netcdf error messages.
ncmex ( 'setopts', 0 );


%
% Assume we don't create the file or variable until we know we have to.
create_file = 0;
create_var = 0;

%
% Must have at least 3 input arguments, no more than 5
if nargin < 3 | nargin==4 | nargin > 5
	fprintf ( 2, '%s:  number of input arguments is wrong.\n', mfilename );
	help nc_varput;
	return;
end





%
% One optional output argument.
if ( nargout > 1 )
	fprintf ( 2, '%s:  at most 1 output argument is allowed.\n', mfilename );
	help nc_varput;
	return;
end


%
% open the file.  Create if it does not exist.
[ncid, status] = ncmex('open', ncfile, 'write');
if (ncid == -1)

	create_file = 1;
	[ncid, status] = ncmex('create', ncfile, 'write');
	if ( ncid == -1 )
		fprintf ( 2, '%s:  ''create'' failed on %s.\n', mfilename, ncfile );
		return;
	end

	status = ncmex ( 'endef', ncid );
	if ( status == -1 )
		fprintf ( 2, '%s:  ''endef'' operation failed on %s.\n', mfilename, ncfile );
		ncmex ( 'close', ncid );
		return;
	end

end


%
% check to see if the variable already exists.  If it does,
% then we should only have 5 input arguments.
[varid, rcode] = ncmex('VARID', ncid, variable );
if ( varid == -1 )

	fprintf ( 2, '%s:  variable %s does not exist.  Use NCMEX interface directly, NetCDF MATLAB toolbox,  or command line utilities to create it.\n' , mfilename, variable );	
	ncmex ( 'close', ncid );
	return;

end



%
% Get variable information.
[var_name,var_type,nvdims,var_dim,natts, status]=ncmex('varinq',ncid,varid);
if status == -1 
	fprintf ( 2, '%s:  ''varinq'' operation failed on %s, variable.\n', mfilename, ncfile, variable );
	ncmex ( 'close', ncid );
	return;
end


%
% Must determine how big the target variable is in order to properly set
% the start and count arguments.
for n=1:nvdims,
    dimid=var_dim(n);
    [dim_name,dim_size,rcode]=ncmex('diminq',ncid,dimid);
    nc_count(n)=dim_size;
end

nc_start = zeros(size(nc_count));




%
% If restrictions were specified, set them here.
if ( nargin > 3 )
	
	if ( length(start) ~= length(count) )
		fprintf ( 2, '%s:  start and count arguments do not have the same length.\n', mfilename );
		ncmex ( 'close', ncid );
		return
	end

	for n=1:length(count)
		nc_start(n)=start(n);
		nc_count(n)=count(n);
	end

end


%
% If only 3 inputs, then START is [0,..] and COUNT is 
% the size of the data.  
if ( nargin ==3 )
	for j = 1:ndims(data)
		nc_start(j) = 0;
		nc_count(j) = size(data,j);
	end
end






%
% Get the fill value, if present.
% Turn any NaNs present in the data into fill values before
% writing. 
[scale, status] = ncattget( ncfile, variable, 'scale_factor');  
if status == -1 

	%
	% if this failed, then there was no scale attribute. 
	% Assume 1.0
	scale = 1;

end
[offset, status] = ncattget(ncfile, variable, 'add_offset');  
if status == -1 

	%
	% if this failed, then there was no scale attribute. 
	% Assume 1.0
	offset = 0;

end
	
[fv, status] = ncattget(ncfile, variable, '_FillValue');
if status ~= -1 
	sc_fv = fv*scale+offset;
	nanind = find(isnan(data));
	data(nanind) = sc_fv;
end




%
% write the data
weird_netcdf_data = permute(data,fliplr([1:ndims(data)]));
status = ncmex('VARPUT', ncid, varid, nc_start, nc_count, weird_netcdf_data, 1);
if ( status == -1 )
	fprintf ( 2, '%s:  ''varput'' operation failed on %s, %s.\n', mfilename, variable, ncfile );
	ncmex ( 'close', ncid );
	return;
end

status = ncmex ( 'close', ncid );

%
% If we got this far, then we must have succeeded.
status = 1;


return
